@@ -8,7 +8,7 @@ fn button_example() -> impl View {
 }
 
 fn slider_example() -> impl View {
-    hstack((caption("slider"), state(0.5, |s| hslider(s))))
+    hstack((caption("slider"), state(|| 0.5, |s| hslider(s))))
 }
 
 fn caption(s: &str) -> impl View {
@@ -18,21 +18,24 @@ fn caption(s: &str) -> impl View {
 fn knob_example() -> impl View {
     hstack((
         caption("knob"),
-        state(0.5, |s| knob(s).size([30.0, 30.0]).padding(Auto)),
+        state(|| 0.5, |s| knob(s).size([30.0, 30.0]).padding(Auto)),
     ))
 }
 
 fn toggle_example() -> impl View {
     hstack((
         caption("toggle"),
-        state(false, |b| toggle(b).size([30.0, 30.0]).padding(Auto)),
+        state(|| false, |b| toggle(b).size([30.0, 30.0]).padding(Auto)),
     ))
 }
 
 fn text_editor_example() -> impl View {
     hstack((
         caption("text_editor"),
-        state("edit me".to_string(), |txt| text_editor(txt).padding(Auto)),
+        state(
+            || "edit me".to_string(),
+            |txt| text_editor(txt).padding(Auto),
+        ),
     ))
 }
 
