@@ -68,7 +68,7 @@ where
         })
     }
 
-    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut StateMap) {
+    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut Vec<ViewID>) {
         self.child.gc(id.child(&0), cx, map);
     }
 
@@ -295,7 +295,7 @@ where
         });
     }
 
-    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut StateMap) {
+    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut Vec<ViewID>) {
         self.child.gc(id.child(&0), cx, map)
     }
 
