@@ -20,13 +20,13 @@ where
             EventKind::TouchBegin { .. } => {
                 if let Some(_) = self.hittest(vid, event.position, cx, vger) {
                     cx.focused_id = Some(vid);
-                    cx.dirty.lock().unwrap().dirty = true;
+                    set_state_dirty();
                 }
             }
             EventKind::Key(KeyPress::Escape, _) => {
                 if cx.focused_id == Some(vid) {
                     cx.focused_id = None;
-                    cx.dirty.lock().unwrap().dirty = true;
+                    set_state_dirty();
                 }
             }
             _ => (),
@@ -56,7 +56,7 @@ where
         (self.func)(Some(id) == cx.focused_id).commands(id.child(&0), cx, cmds)
     }
 
-    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut StateMap) {
+    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut Vec<ViewID>) {
         (self.func)(Some(id) == cx.focused_id).gc(id.child(&0), cx, map)
     }
 
