@@ -3,11 +3,17 @@
 mod view;
 pub use view::*;
 
+mod viewid;
+pub use viewid::*;
+
+mod event;
+pub use event::*;
+
 mod binding;
 pub use binding::*;
 
-mod state;
-pub use state::*;
+mod gstate;
+pub use gstate::*;
 
 mod text;
 pub use text::*;
@@ -101,7 +107,6 @@ use vger::*;
 use tao::{
     accelerator::Accelerator,
     dpi::PhysicalSize,
-    event,
     event::{ElementState, WindowEvent},
     event_loop::{ControlFlow, EventLoop},
     keyboard::ModifiersState,
@@ -111,6 +116,9 @@ use tao::{
 
 use std::env;
 
+#[macro_use]
+extern crate lazy_static;
+
 // See https://rust-lang.github.io/api-guidelines/future-proofing.html
 pub(crate) mod private {
     pub trait Sealed {}
@@ -302,14 +310,18 @@ pub fn rui(view: impl View + 'static) {
     };
     surface.configure(&device, &config);
 
+    *GLOBAL_EVENT_LOOP_PROXY.lock().unwrap() = Some(event_loop.create_proxy());
+
     let mut vger = VGER::new(&device, wgpu::TextureFormat::Bgra8UnormSrgb);
-    let mut cx = Context::new(Some(event_loop.create_proxy()), window);
+    let mut cx = Context::new(Some(window));
     let mut mouse_position = LocalPoint::zero();
 
     let mut commands = Vec::new();
     view.commands(cx.root_id, &mut cx, &mut commands);
     let mut command_map = HashMap::new();
     cx.window
+        .as_ref()
+        .unwrap()
         .set_menu(Some(build_menubar(&commands, &mut command_map)));
 
     let mut modifiers = ModifiersState::default();
@@ -327,14 +339,14 @@ pub fn rui(view: impl View + 'static) {
         *control_flow = ControlFlow::Wait;
 
         match event {
-            event::Event::WindowEvent {
+            tao::event::Event::WindowEvent {
                 event: WindowEvent::CloseRequested,
                 ..
             } => {
                 println!("The close button was pressed; stopping");
                 *control_flow = ControlFlow::Exit
             }
-            event::Event::WindowEvent {
+            tao::event::Event::WindowEvent {
                 event:
                     WindowEvent::Resized(size)
                     | WindowEvent::ScaleFactorChanged {
@@ -347,12 +359,12 @@ pub fn rui(view: impl View + 'static) {
                 config.width = size.width.max(1);
                 config.height = size.height.max(1);
                 surface.configure(&device, &config);
-                cx.window.request_redraw();
+                cx.window.as_ref().unwrap().request_redraw();
             }
-            event::Event::UserEvent(_) => {
+            tao::event::Event::UserEvent(_) => {
                 // println!("received user event");
             }
-            event::Event::MainEventsCleared => {
+            tao::event::Event::MainEventsCleared => {
                 // Application update code.
 
                 // Queue a RedrawRequested event.
@@ -360,7 +372,7 @@ pub fn rui(view: impl View + 'static) {
                 // You only need to call this if you've determined that you need to redraw, in
                 // applications which do not always need to. Applications that redraw continuously
                 // can just render here instead.
-                if cx.dirty.lock().unwrap().dirty {
+                if is_state_dirty() {
                     // Have the commands changed?
                     let mut new_commands = Vec::new();
                     view.commands(cx.root_id, &mut cx, &mut new_commands);
@@ -370,15 +382,21 @@ pub fn rui(view: impl View + 'static) {
                         commands = new_commands;
 
                         command_map.clear();
-                        cx.window
+                        cx.window.as_ref().unwrap()
                             .set_menu(Some(build_menubar(&commands, &mut command_map)));
                     }
 
                     // Clean up state.
-                    let mut new_map = StateMap::new();
-                    view.gc(cx.root_id, &mut cx, &mut new_map);
-                    // println!("collected {} states", cx.state_map.len() - new_map.len());
-                    cx.state_map = new_map;
+                    let mut keep = vec![];
+                    view.gc(cx.root_id, &mut cx, &mut keep);
+                    {
+                        let mut map = GLOBAL_STATE_MAP.lock().unwrap();
+                        let mut new_map = StateMap::new();
+                        for id in keep {
+                            new_map.insert(id, map[&id].clone());
+                        }
+                        *map = new_map;
+                    }
 
                     // Get a new accesskit tree.
                     let mut nodes = vec![];
@@ -397,12 +415,12 @@ pub fn rui(view: impl View + 'static) {
                         // println!("access nodes unchanged");
                     }
 
-                    cx.window.request_redraw();
+                    cx.window.as_ref().unwrap().request_redraw();
 
-                    cx.dirty.lock().unwrap().dirty = false;
+                    clear_state_dirty();
                 }
             }
-            event::Event::RedrawRequested(_) => {
+            tao::event::Event::RedrawRequested(_) => {
                 // Redraw the application.
                 //
                 // It's preferable for applications that do not render continuously to render in
@@ -421,16 +439,20 @@ pub fn rui(view: impl View + 'static) {
                     }
                 };
 
-                let window_size = cx.window.inner_size();
-                let scale = cx.window.scale_factor() as f32;
+                let window_size = cx.window.as_ref().unwrap().inner_size();
+                let scale = cx.window.as_ref().unwrap().scale_factor() as f32;
                 // println!("window_size: {:?}", window_size);
                 let width = window_size.width as f32 / scale;
                 let height = window_size.height as f32 / scale;
 
                 vger.begin(width, height, scale);
 
+                // Disable dirtying the state during layout and rendering
+                // to avoid constantly re-rendering if some state is saved.
+                ENABLE_DIRTY.with(|enable| *enable.borrow_mut() = false);
                 view.layout(cx.root_id, [width, height].into(), &mut cx, &mut vger);
                 view.draw(cx.root_id, &mut cx, &mut vger);
+                ENABLE_DIRTY.with(|enable| *enable.borrow_mut() = true);
 
                 let texture_view = frame
                     .texture
@@ -453,20 +475,20 @@ pub fn rui(view: impl View + 'static) {
 
                 frame.present();
             }
-            event::Event::WindowEvent {
+            tao::event::Event::WindowEvent {
                 event: WindowEvent::MouseInput { state, .. },
                 ..
             } => {
                 match state {
                     ElementState::Pressed => {
-                        let event = view::Event {
+                        let event = Event {
                             kind: EventKind::TouchBegin { id: 0 },
                             position: mouse_position,
                         };
                         view.process(&event, cx.root_id, &mut cx, &mut vger)
                     }
                     ElementState::Released => {
-                        let event = view::Event {
+                        let event = Event {
                             kind: EventKind::TouchEnd { id: 0 },
                             position: mouse_position,
                         };
@@ -475,47 +497,47 @@ pub fn rui(view: impl View + 'static) {
                     _ => {}
                 };
             }
-            event::Event::WindowEvent {
+            tao::event::Event::WindowEvent {
                 event: WindowEvent::CursorMoved { position, .. },
                 ..
             } => {
-                let scale = cx.window.scale_factor() as f32;
+                let scale = cx.window.as_ref().unwrap().scale_factor() as f32;
                 mouse_position = [
                     position.x as f32 / scale,
                     (config.height as f32 - position.y as f32) / scale,
                 ]
                 .into();
-                let event = view::Event {
+                let event = Event {
                     kind: EventKind::TouchMove { id: 0 },
                     position: mouse_position,
                 };
                 view.process(&event, cx.root_id, &mut cx, &mut vger)
             }
-            event::Event::WindowEvent {
+            tao::event::Event::WindowEvent {
                 event: WindowEvent::KeyboardInput { event, .. },
                 ..
             } => {
                 if event.state == ElementState::Pressed {
-                    let event = view::Event {
+                    let event = Event {
                         kind: EventKind::Key(event.logical_key, modifiers),
                         position: mouse_position,
                     };
                     view.process(&event, cx.root_id, &mut cx, &mut vger)
                 }
             }
-            event::Event::WindowEvent {
+            tao::event::Event::WindowEvent {
                 event: WindowEvent::ModifiersChanged(mods),
                 ..
             } => {
                 modifiers = mods;
                 // println!("modifiers changed: {:?}", modifiers);
             }
-            event::Event::MenuEvent { menu_id, .. } => {
+            tao::event::Event::MenuEvent { menu_id, .. } => {
                 //println!("menu event");
 
                 if let Some(command) = command_map.get(&menu_id) {
                     //println!("found command {:?}", command);
-                    let event = view::Event {
+                    let event = Event {
                         kind: EventKind::Command(command.clone()),
                         position: mouse_position,
                     };
@@ -539,28 +561,26 @@ mod tests {
         });
     }
 
-    // #[test]
-    // fn test_state2() {
-    //     let mut cx = Context::new(None);
-    //     let v = counter(42);
-    //     v.print(ViewID::default(), &mut cx);
-    // }
+    #[test]
+    fn test_state2() {
+        let mut cx = Context::new(None);
+        let v = counter(42);
+        v.print(ViewID::default(), &mut cx);
+    }
 
     fn counter(start: usize) -> impl View {
         state(
             move || start,
-            |count: State<usize>| {
+            |count| {
                 let count2 = count.clone();
                 let value_string = format!("value: {:?}", count.get());
                 vstack((
                     text(value_string.as_str()),
                     button(text("increment"), move || {
-                        let value = count.get();
-                        count.set(value + 1);
+                        count.with_mut(|value| *value += 1);
                     }),
                     button(text("decrement"), move || {
-                        let value = count2.get();
-                        count2.set(value - 1);
+                        count2.with_mut(|value| *value -= 1);
                     }),
                 ))
             },
@@ -574,18 +594,16 @@ mod tests {
         let count2 = count.clone();
         vstack((
             button(text("increment"), move || {
-                let value = count.get();
-                count.set(value + 1);
+                count.with_mut(|value| *value += 1);
             }),
             button(text("decrement"), move || {
-                let value = count2.get();
-                count2.set(value - 1);
+                count2.with_mut(|value| *value -= 1);
             }),
         ))
     }
 
     #[test]
     fn test_binding() {
-        let _ = state(|| 42, |count: State<usize>| counter3(count));
+        let _ = state(|| 42, |count| counter3(count));
     }
 }
