@@ -105,7 +105,7 @@ where
         }
     }
 
-    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut StateMap) {
+    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut Vec<ViewID>) {
         for child in &self.ids {
             ((self.func)(child)).gc(id.child(child), cx, map)
         }
