@@ -8,6 +8,9 @@ pub trait Modifiers: View + Sized {
     /// Calls a function in response to a tap.
     fn tap<F: Fn() + 'static>(self, f: F) -> Tap<Self, F>;
 
+    
+    fn tap2<F: FnMut()>(self, f: F) -> Tap2<Self, F>;
+
     /// Puts a view behind another. The background view inherits the size of the view.
     fn background<BG: View + 'static>(self, background: BG) -> Background<Self, BG>;
 
@@ -51,6 +54,9 @@ impl<V: View + 'static> Modifiers for V {
     fn tap<F: Fn() + 'static>(self, f: F) -> Tap<Self, F> {
         Tap::new(self, f)
     }
+    fn tap2<F: FnMut()>(self, f: F) -> Tap2<Self, F> {
+        Tap2::new(self, f)
+    }
     fn background<BG: View + 'static>(self, background: BG) -> Background<Self, BG> {
         Background::new(self, background)
     }
