@@ -47,7 +47,7 @@ where
         self.child.commands(id.child(&0), cx, cmds)
     }
 
-    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut StateMap) {
+    fn gc(&self, id: ViewID, cx: &mut Context, map: &mut Vec<ViewID>) {
         self.child.gc(id.child(&0), cx, map)
     }
 
