@@ -1,25 +1,10 @@
 pub use crate::*;
-use std::cell::RefCell;
-use std::rc::Rc;
-
-struct TextEditorGlyphInfo {
-    glyph_rects: Vec<LocalRect>,
-    lines: Vec<LineMetrics>,
-}
-
-impl TextEditorGlyphInfo {
-    fn new() -> Self {
-        Self {
-            glyph_rects: vec![],
-            lines: vec![],
-        }
-    }
-}
 
 #[derive(Clone)]
 struct TextEditorState {
     cursor: usize,
-    glyph_info: Rc<RefCell<TextEditorGlyphInfo>>,
+    glyph_rects: Vec<LocalRect>,
+    lines: Vec<LineMetrics>,
 }
 
 impl TextEditorState {
@@ -37,7 +22,7 @@ impl TextEditorState {
 
     fn find_line(&self) -> usize {
         let mut i = 0;
-        for line in &self.glyph_info.borrow().lines {
+        for line in &self.lines {
             if self.cursor >= line.glyph_start && self.cursor < line.glyph_end {
                 break;
             }
@@ -65,26 +50,22 @@ impl TextEditorState {
     }
 
     fn down(&mut self) {
-        let info = self.glyph_info.borrow();
-        let rects = &info.glyph_rects;
-        let p = rects[self.cursor].center();
+        let p = self.glyph_rects[self.cursor].center();
 
         let line = self.find_line() + 1;
-        if line < self.glyph_info.borrow().lines.len() {
-            let metrics = self.glyph_info.borrow().lines[line];
-            self.cursor = self.closest_in_range(p, metrics.glyph_start..metrics.glyph_end, rects);
+        if line < self.lines.len() {
+            let metrics = self.lines[line];
+            self.cursor = self.closest_in_range(p, metrics.glyph_start..metrics.glyph_end, &self.glyph_rects);
         }
     }
 
     fn up(&mut self) {
-        let info = self.glyph_info.borrow();
-        let rects = &info.glyph_rects;
-        let p = rects[self.cursor].center();
+        let p = self.glyph_rects[self.cursor].center();
 
         let line = self.find_line();
         if line > 0 {
-            let metrics = self.glyph_info.borrow().lines[line - 1];
-            self.cursor = self.closest_in_range(p, metrics.glyph_start..metrics.glyph_end, rects);
+            let metrics = self.lines[line - 1];
+            self.cursor = self.closest_in_range(p, metrics.glyph_start..metrics.glyph_end, &self.glyph_rects);
         }
     }
 
@@ -125,7 +106,8 @@ impl TextEditorState {
     fn new() -> Self {
         Self {
             cursor: 0,
-            glyph_info: Rc::new(RefCell::new(TextEditorGlyphInfo::new())),
+            glyph_rects: vec![],
+            lines: vec![],
         }
     }
 }
@@ -140,16 +122,12 @@ where
     B: Binding<String>,
 {
     fn body(&self) -> impl View {
-        let text = self.text.clone();
+        let text = self.text;
         focus(move |has_focus| {
-            let text = text.clone();
             state(
                 || TextEditorState::new(),
                 move |state| {
-                    let text = text.clone();
-                    let text2 = text.clone();
                     let cursor = state.with(|s| s.cursor);
-                    let state2 = state.clone();
                     canvas(move |rect, vger| {
                         vger.translate([0.0, rect.height()]);
                         let font_size = 18;
@@ -170,12 +148,11 @@ where
                             );
                         }
 
-                        state2.get().glyph_info.borrow_mut().glyph_rects = rects;
-                        state2.get().glyph_info.borrow_mut().lines = lines;
+                        state.with_mut(|s| { s.glyph_rects = rects; s.lines = lines; });
                     })
                     .key(move |k| {
                         if has_focus {
-                            state.with_mut(|s| s.key(&k, &text2))
+                            state.with_mut(|s| s.key(&k, &text))
                         }
                     })
                 },
